import webbrowser

from functools import partial
from io import StringIO
from tempfile import gettempdir

from line_profiler import LineProfiler


class WebRender:
    def __init__(self, profiler: LineProfiler.print_stats):
        self.profiler = partial(profiler, output_unit=0.001, stripzeros=False)

    def _get_content(self) -> str:
        stream = StringIO()
        self.profiler(stream)
        return stream.getvalue()

    def render(self) -> None:
        path = f"{gettempdir()}/cpu_profile.html"

        with open(path, "w") as f:
            html = self.get_html()
            f.write(html)

        webbrowser.open(f"file://{path}")

    def get_html(self) -> str:
        content = self._get_content()
        return f'<html><head><meta charset="UTF-8"/></head><pre>{content}</pre></html>'



import atexit

from line_profiler import LineProfiler

from .web import WebRender


class CPUProfiler(LineProfiler):
    def __init__(self):
        super(LineProfiler, self).__init__()
        self.stream = WebRender(self.print_stats)

    def auto_report(self) -> None:
        """
        Auto open web report in browser after execute script.
        """

        atexit.register(self.open_report)

    def open_report(self) -> None:
        """
        Open web report in browser.
        """
        self.stream.render()

    def get_report(self) -> str:
        """
        Get source html for web report.
        """
        return self.stream.get_html()


cpu = CPUProfiler()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pprof']

package_data = \
{'': ['*']}

install_requires = \
['line-profiler>=3.5.1,<4.0.0']

setup_kwargs = {
    'name': 'pprof',
    'version': '0.1.2',
    'description': 'Python profiling tool',
    'long_description': '<p align="center">\n  <a href="https://github.com/mirecl/pprof"><img src="https://github.com/mirecl/pprof/blob/master/examples/report.png?raw=true" alt="pprof"></a>\n</p>\n\n[![PyPI](https://img.shields.io/pypi/v/pprof)](https://pypi.org/project/pprof/)\n[![Downloads](https://pepy.tech/badge/pprof)](https://pepy.tech/project/pprof)\n[![Code Style](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n[![GitHub](https://img.shields.io/github/license/mirecl/pprof)](https://github.com/mirecl/pprof/blob/master/LICENSE)\n[![Tests](https://github.com/mirecl/pprof/actions/workflows/tests.yaml/badge.svg)](https://github.com/mirecl/pprof/actions/workflows/tests.yaml)\n[![codecov](https://codecov.io/gh/mirecl/pprof/branch/master/graph/badge.svg?token=UFDA1JG40A)](https://codecov.io/gh/mirecl/pprof)\n[![python version](https://img.shields.io/pypi/pyversions/pprof.svg)](https://pypi.org/project/pprof/)\n\n## Installing\n\n```sh\npip install pprof\n```\n\nor\n\n```sh\npoetry add pprof\n```\n\n## A Simple Example\n\n```python\nfrom typing import List\nfrom pprof import cpu\n\ncpu.auto_report()\n\n@cpu\ndef run(arr: List) -> float:\n    tmp = []\n    for row in arr:\n        if row % 3 == 0:\n            tmp.append(row)\n    result = (sum(tmp*100) + len(arr)) / len(tmp)\n    return result\n\nrun(list(range(100000)))\n```\n\n```sh\n(venv) python run.py\n```\n\n## Links\n\n+ **line_profiler** (<https://github.com/pyutils/line_profiler>)\n',
    'author': 'mirecl',
    'author_email': 'grazhdankov@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.2,<4.0',
}


setup(**setup_kwargs)
